/*
 * Functions to speed up line reading from .ped files by reading whole lines
 * at a time
 */

#ifndef _READLINE_H
#define	_READLINE_H

#include <stdio.h>

// On linux we have a nice fast function to read lines
#ifdef __USE_GNU

#define HAVE_GETLINE 1

#else

#undef HAVE_GETLINE

#endif

using namespace std;

extern FILE *in;

char *line, *buf;
size_t line_s, buf_s;
int line_pos, buf_read;

/**
 * Initialize line buffer
 */
void initLine()
{
    line_s = 100000;
    line_pos = 0;
    line = (char*) malloc(line_s);

    buf_s = 100000;
    buf_read = 0;
    buf = (char*) malloc(buf_s);
}


/**
 * Free the space used
 */
void freeLine()
{
    free(line);
    free(buf);
}

/**
 * Read the next line and store it to global variable line
 */
int readLine()
{

#ifdef HAVE_GETLINE

    int a = getline(&line, (size_t *) &line_s, in);
    line_pos = 0;

    return a;

#else

    int n = 0;
    int lp = 0;
    int p = buf_read;
    line_pos = 0;

    while(1) {
	n = fread(buf,1,buf_s,in);

	if(n==0) return -1;
	p = 0;

	while(p < buf_s) {

	    if(lp >= line_s) {
		line_s *=2;
		line = (char*)realloc(line,line_s);
	    }

	    line[lp++] = buf[ p++ ];

	    if(buf[p-1] == '\n') {
		long ofs;
		ofs = -((long)n-p);
		buf_read = p; line[lp++] = 0;
		fseek(in, ofs, SEEK_CUR);
		return 0;
	    }
	}

	if(n==0 && lp == 0) return -1;
    }

#endif
}




#if 0

#define INITIAL_BUFFER_SIZE 5000

int getlinexxx(char **line, size_t *len, FILE *stream)
{
    size_t size = 0, last = 0;

    do {
	size += BUFSIZ;
	*line = (char*)realloc(*line, size);
	fgets(*line + last, size, stream);
	*len = strlen(*line);
	last = *len - 1;
    }while (!feof(stream) && *line[last] != '\n');
    return 0;
}


#endif

#endif
